<?php
/*
  Plugin Name: ERIP Payment Gateway
  Plugin URI: 
  Description: Allows you to use ERIP payment gateway with the WooCommerce plugin.
  Version: 1.0
  Author: Tatsiana Stadub
  Author URI: http://easypay.by
 */

if (!defined('ABSPATH')) exit; // Exit if accessed directly

/* Add a custom payment class to WC
  ------------------------------------------------------------ */

add_action('plugins_loaded', 'load_erip', 0);

function load_erip() {
	if (!class_exists('WC_Payment_Gateway')) {
		return; 
	}
	if(class_exists('WC_ERIP')) {
		return;
	}

class WC_ERIP extends WC_Payment_Gateway {

	public function __construct() {
		$plugin_dir = plugin_dir_url(__FILE__);
		global $woocommerce;
		$this->id      = "ЕРИП";
		$this->liveurl = 'https://ssl.easypay.by/weborder/';

		//Загрузка настроек
		$this->init_form_fields();
		$this->init_settings();

		//Определение переменных
		$this->title       = $this->get_option('title');
		$this->mer_no      = $this->get_option('mer_no');
		$this->web_key     = $this->get_option('web_key');
		$this->description = $this->get_option('description');
		$this->expires     = $this->get_option('expires');
		$this->comment     = $this->get_option('comment');

		add_action('woocommerce_receipt_' . $this->id, array($this, 'receipt_page'));

		//Сохранение данных в админке
		add_action('woocommerce_update_options_payment_gateways_' . $this->id, array($this, 'process_admin_options'));

		//Вызов функции для обработки урлов возврата
		add_action( 'woocommerce_api_wc_erip', array( $this, 'check_ipn_response' ) );
	}

	/**
	* Admin Panel Options 
	* - Options for bits like 'title' and availability on a country-by-country basis
	*
	* @since 0.1
	**/
	public function admin_options() {
		?>
		<h3><?php _e('ЕРИП', 'woocommerce'); ?></h3>
		<p><?php _e('Настройка приема электронных платежей через ЕРИП.', 'woocommerce'); ?></p>
		<table class="form-table">
			<?php		
				$this->generate_settings_html();
			?>
		</table>
		<?php
	}

  /**
  * Initialise Gateway Settings Form Fields
  *
  * @access public
  * @return void
  */
	function init_form_fields(){
		$this->form_fields = array(
			'enabled' => array(
				'title'   => __('Включить/Выключить', 'woocommerce'),
				'type'    => 'checkbox',
				'label'   => __('Включен', 'woocommerce'),
				'default' => 'no'
			),
			'title' => array(
				'title'       => __('Название', 'woocommerce'),
				'type'        => 'text', 
				'description' => __('Это название, которое пользователь видит во время оплаты.', 'woocommerce'), 
				'default'     => __('ЕРИП', 'woocommerce')
			),
			'mer_no' => array(
				'title'       => __('Номер Поставщика', 'woocommerce'),
				'type'        => 'text',
				'description' => __('Пожалуйста, введите номер Поставщика.', 'woocommerce'),
				'default'     => ''
			),
			'web_key' => array(
				'title'       => __('Ключ для подписи счетов', 'woocommerce'),
				'type'        => 'text',
				'description' => __('Ключ для подписи счетов web_key.', 'woocommerce'),
				'default'     => ''
			),
			'expires' => array(
				'title'       => __('Срок действия счета', 'woocommerce'),
				'type'        => 'text',
				'description' => __('Число от 1 до 30, если период задан в днях или от 600 до 3600*24, если период задан в секундах.', 'woocommerce'),
				'default'     => '3'
			),
			'description' => array(
				'title'       => __('Описание метода оплаты', 'woocommerce'),
				'type'        => 'textarea',
				'description' => __('Описание метода оплаты, которое клиент будет видеть на вашем сайте.', 'woocommerce'),
				'default'     => 'Оплата с помощью ЕРИП.'
			),
			'comment' => array(
				'title'       => __('Комментарий счета', 'woocommerce'),
				'type'        => 'text',
				'description' => __('Краткий комментарий счета. Формируется Поставщиком (не более 50 символов).', 'woocommerce'),
				'default'     => ''
			),

		);
	}
	//Подготовка формы c данными
	public function generate_form($order_id){
		global $woocommerce;
		$order = new WC_Order($order_id);
		
		$action_adr		= $this->liveurl;
		$out_summ		= $order->order_total;
		$hash			= md5($this->mer_no . $this->web_key . $order_id . $out_summ);
		$shop_page_url	= get_site_url();
		$success_url	= $shop_page_url.'/?wc-api=wc_erip&erip=success';
		$cancel_url		= $shop_page_url.'/?wc-api=wc_erip&erip=fail';

		$args = array(
			'EP_MerNo'       => $this->mer_no,
			'EP_OrderNo'     => $order_id,
			'EP_Sum'         => $out_summ,
			'EP_Expires'     => $this->expires,
			'EP_Comment'     => $this->comment,
			'EP_OrderInfo'   => "Заказ №$order_id",
			'EP_Hash'        => $hash,
			'EP_Success_URL' => $success_url,
			'EP_Cancel_URL'  => $cancel_url,
			'EP_PayType'	 => "PT_ERIP",
			'EP_Encoding'	 => "utf-8",
		);
		$args_array = array();
		foreach ($args as $key => $value){
			$args_array[] = '<input type="hidden" name="' . $key . '" value="' . $value . '" />';
		}
		$woocommerce->cart->empty_cart();//Очищение корзины

		return
			'<form action="' . esc_url($action_adr) . '" method="POST" id="erip_payment_form" >' . "\n".
			implode("\n", $args_array) . '<input type="submit" class="button alt" id="submit_erip_payment_form" value="' . __('Оплатить', 'woocommerce') . '" /> <a class="button cancel" href="' . $order->get_cancel_order_url() . '">' . __('Отказаться от оплаты & вернуться в корзину', 'woocommerce') . '</a>' . "\n" . '</form>';
		
	}

	/**
	 * Process the payment and return the result
	 **/
	function process_payment($order_id){
		$order = new WC_Order($order_id);	
		return array(
			'result' => 'success',
			'redirect'	=> add_query_arg('order', $order->id, add_query_arg('key', $order->order_key, get_permalink(woocommerce_get_page_id('pay'))))
		);
	}

	/**
	* receipt_page
	**/
	function receipt_page($order){
		echo '<p>' . __('Спасибо за Ваш заказ, пожалуйста, нажмите кнопку ниже, чтобы оплатить.', 'woocommerce') . '</p>';
		echo $this->generate_form($order);
	}

	/**
	* Check Response
	**/
	function check_ipn_response() {
		global $woocommerce;
		
		if (isset($_GET['result']) && $_GET['result'] == 'notify') {//Нотификация		
			$order_mer_code		= isset($_POST['order_mer_code']) ? $_POST['order_mer_code'] : '';
			$sum				= isset($_POST['sum']) ? $_POST['sum'] : '';
			$mer_no				= isset($_POST['mer_no']) ? $_POST['mer_no'] : '';
			$card				= isset($_POST['card']) ? $_POST['card'] : '';
			$notify_signature	= isset($_POST['notify_signature']) ? $_POST['notify_signature'] : '';
			$purch_date			= isset($_POST['purch_date']) ? $_POST['purch_date'] : '';
            $web_key			= $this->web_key;
			
			$sign = md5($order_mer_code . $sum . $mer_no . $card . $purch_date . $web_key);

            $order = new WC_Order($order_mer_code);

			if ($notify_signature == $sign) { // проверка правильности подписи
                $order->update_status('completed', __('Счет успешно оплачен', 'woocommerce'));
                $status = 'OK | payment received';
                header("HTTP/1.0 200 OK");
                print $status;
			} else { //подписи не совпадают
                $status = 'FAILED | wrong notify signature'; 
                header("HTTP/1.0 400 Bad Request");
                print $status;
            }
			exit;
		}else if (isset($_GET['erip']) AND $_GET['erip'] == 'success'){//Успешное завершения операции
			$inv_id = $_REQUEST['EP_OrderNo'];
			$order = new WC_Order($inv_id);
			$message = "<div style='color:#00a12d; font-size:16px; font-weight:bold;'>Счет добавлен в систему ЕРИП для оплаты.</div>
						<div style='color:#00a12d; font-size:16px; font-weight:bold;'>Номер заказа для оплаты: $inv_id</div>";
			wc_add_notice($message, 'success' );
			wp_redirect( $this->get_return_url($order));
		}else if (isset($_GET['erip']) AND $_GET['erip'] == 'fail'){//Ошибка в операции
			$inv_id = $_REQUEST['EP_OrderNo'];
			$order = new WC_Order($inv_id);
			$order->update_status('failed', __('Платеж не оплачен', 'woocommerce'));
			$message = "<div style='color:#D1001D; font-size:16px; font-weight:bold;'>Ошибка оплаты счета<div>";
			wc_add_notice($message, 'error' );
			wp_redirect($order->get_cancel_order_url());
			exit;
		}
	}
}
	add_filter( 'woocommerce_payment_gateways', 'add_WC_ERIP' );
	function add_WC_ERIP( $methods ){
		$methods[] = 'wc_erip';
		return $methods;
	}
}

?>